/*
Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.

The MySQL Connector/C++ is licensed under the terms of the GPLv2
<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
MySQL Connectors. There are special exceptions to the terms and
conditions of the GPLv2 as it is applied to this software, see the
FLOSS License Exception
<http://www.mysql.com/about/legal/licensing/foss-exception.html>.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
*/



#ifndef __TESTCONTAINER_H_
#define __TESTCONTAINER_H_

#include "../common/ccppTypes.h"
#include <boost/noncopyable.hpp>


namespace testsuite
{

  class Test;

  namespace Private
  {
    class TestContainer
    {
    protected:

    class StorableTest : public boost::noncopyable
      {
        StorableTest(){}

        Test *        test;

      public:

        virtual ~StorableTest();

        StorableTest( Test & test2decorate );

         /*Test & operator * ();
        Test * operator ->();*/

        Test * get();

      };
    };
  }
}


#endif  // ifndef __TESTCONTAINER_H_
