/*
Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.

The MySQL Connector/C++ is licensed under the terms of the GPLv2
<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
MySQL Connectors. There are special exceptions to the terms and
conditions of the GPLv2 as it is applied to this software, see the
FLOSS License Exception
<http://www.mysql.com/about/legal/licensing/foss-exception.html>.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
*/



#include "cppconn/exception.h"

#include "../BaseTestFixture.h"

/**
 *
 * @author Mark Matthews
 * @version $Id: TransactionTest.java,v 1.1.2.1 2005/05/13 18:58:37 mmatthews
 *          Exp $
 */

namespace testsuite
{
namespace simple
{
  class TransactionTest : public BaseTestFixture
  {
  private:
    typedef BaseTestFixture super;
  // ---------------------------------------------

    static const double DOUBLE_CONST;

/* throws sql::DbcException */

    void createTestTable() ;

protected:

  public:
    TEST_FIXTURE( TransactionTest )
    {
      TEST_CASE( testTransaction );
    }


    /* throws std::exception * */
    void setUp();

  /**
	 * DOCUMENT ME!
	 *
	 * @throws sql::SQLException
	 *             DOCUMENT ME!
	 */
  /* throws sql::SQLException */

    void testTransaction();
  };

REGISTER_FIXTURE(TransactionTest);

}
}
